@@ -1,6 +1,7 @@
 from libifstate.exception import LinkDuplicate
 from libifstate.link.base import ethtool_path, Link
 from libifstate.address import Addresses
+from libifstate.neighbour import Neighbours
 from libifstate.routing import Tables, Rules
 from libifstate.sysctl import Sysctl
 from libifstate.parser import Parser
@@ -18,18 +19,34 @@ except Exception as err:
     if not isinstance(err, netlinkerror_classes):
         raise
 
+try:
+    from libifstate.bpf import libbpf
+except OSError:
+    # ignore missing plugin
+    pass
+except AttributeError:
+    # ignore missing symbols (i.e. libbpf0)
+    pass
+
+try:
+    from libifstate.xdp import XDP
+except ModuleNotFoundError:
+    # ignore missing plugin
+    pass
+
 from libifstate.util import logger, ipr, IfStateLogging
 from libifstate.exception import FeatureMissingError, LinkCircularLinked, LinkNoConfigFound, ParserValidationError
 from ipaddress import ip_network, ip_interface
-from jsonschema import validate, ValidationError
+from jsonschema import validate, ValidationError, FormatChecker
 from copy import deepcopy
 import os
 import pkgutil
 import re
 import json
 import errno
+import logging
 
-__version__ = "1.5.7"
+__version__ = "1.6.0"
 
 
 class IfState():
@@ -37,6 +54,7 @@ class IfState():
         logger.debug('IfState {}'.format(__version__))
         self.links = {}
         self.addresses = {}
+        self.neighbours = {}
         self.ignore = {}
         self.vrrp = {
             'links': [],
@@ -48,23 +66,31 @@ class IfState():
         self.sysctl = Sysctl()
         self.tc = {}
         self.wireguard = {}
+        self.xdp = {}
         self.features = {
             'link': True,
             'sysctl': os.access('/proc/sys/net', os.R_OK),
             'ethtool': not ethtool_path is None,
             'tc': True,
             'wireguard': not globals().get("WireGuard") is None,
+            'bpf': not globals().get("libbpf") is None,
+            'xdp': not globals().get("XDP") is None,
         }
 
         logger.debug('{}'.format(' '.join(sorted(
             [x for x, y in self.features.items() if y]))), extra={'iface': 'features'})
 
+        if self.features['bpf']:
+            if logger.level != logging.DEBUG:
+                # BPF: disable libbpf stderr output
+                libbpf.libbpf_set_print(0)
+
     def update(self, ifstates, soft_schema):
         # check config schema
         schema = json.loads(pkgutil.get_data(
             "libifstate", "../schema/ifstate.conf.schema.json"))
         try:
-            validate(ifstates, schema)
+            validate(ifstates, schema, format_checker=FormatChecker())
         except ValidationError as ex:
             if len(ex.path) > 0:
                 path = ["$"]
@@ -108,6 +134,11 @@ class IfState():
             else:
                 self.addresses[name] = None
 
+            if 'neighbours' in ifstate:
+                self.neighbours[name] = Neighbours(name, ifstate['neighbours'])
+            else:
+                self.neighbours[name] = None
+
             if 'vrrp' in ifstate:
                 ktype = ifstate['vrrp']['type']
                 kname = ifstate['vrrp']['name']
@@ -191,6 +222,12 @@ class IfState():
 
                 self.wireguard[name] = WireGuard(name, ifstate['wireguard'])
 
+            if 'xdp' in ifstate:
+                if not self.features['xdp']:
+                    raise FeatureMissingError("xdp")
+
+                self.xdp[name] = XDP(name, ifstate['xdp'])
+
         # add routing from config
         if 'routing' in ifstates:
             if 'routes' in ifstates['routing']:
@@ -341,6 +378,19 @@ class IfState():
                 else:
                     tc.apply(do_apply)
 
+        if any(not x is None for x in self.xdp.values()):
+            logger.info("\nconfiguring eXpress Data Path...")
+
+            for name, xdp in self.xdp.items():
+                if name in vrrp_ignore:
+                    logger.debug('skipped due to vrrp constraint',
+                                 extra={'iface': name})
+                elif xdp is None:
+                    logger.debug('skipped due to no xdp settings', extra={
+                                 'iface': name})
+                else:
+                    xdp.apply(do_apply)
+
         if any(not x is None for x in self.addresses.values()):
             logger.info("\nconfiguring interface ip addresses...")
             # add empty objects for unhandled interfaces
@@ -363,6 +413,27 @@ class IfState():
         else:
             logger.info("\nno interface ip addressing to be applied")
 
+        if any(not x is None for x in self.neighbours.values()):
+            logger.info("\nconfiguring interface neighbours...")
+            # add empty objects for unhandled interfaces
+            for link in ipr.get_links():
+                name = link.get_attr('IFLA_IFNAME')
+                # skip links on ignore list
+                if not name in self.neighbours and not any(re.match(regex, name) for regex in self.ignore.get('ifname', [])):
+                    self.neighbours[name] = Neighbours(name, [])
+
+            for name, neighbours in self.neighbours.items():
+                if name in vrrp_ignore:
+                    logger.debug('skipped due to vrrp constraint',
+                                 extra={'iface': name})
+                elif neighbours is None:
+                    logger.debug('skipped due to no address settings', extra={
+                                 'iface': name})
+                else:
+                    neighbours.apply(do_apply)
+        else:
+            logger.info("\nno interface neighbours to be applied")
+
         if not self.tables is None:
             self.tables.apply(self.ignore.get('routes', []), do_apply)
 
